@@ -3,7 +3,8 @@
 
 import numpy as np
 
-from numpy.testing import assert_raises, assert_equal
+from numpy.testing import assert_equal
+from pytest import raises as assert_raises
 
 from scipy.io.matlab.miobase import matdims
 
